@@ -3,15 +3,15 @@ from copy import deepcopy
 import numpy as np
 import astropy.io.fits as fits
 import astropy.units as u
-from astropy.coordinates import Angle
 from gammapy.irf import EnergyDependentTablePSF
-from gammapy.maps import Map
+from gammapy.maps import Map, MapAxis, MapCoord, WcsGeom
+from gammapy.utils.random import InverseCDFSampler, get_random_state
 from .psf_kernel import PSFKernel
 
 __all__ = ["make_psf_map", "PSFMap"]
 
 
-def make_psf_map(psf, pointing, geom, max_offset, exposure_map=None):
+def make_psf_map(psf, pointing, geom, exposure_map=None):
     """Make a psf map for a single observation
 
     Expected axes : rad and true energy in this specific order
@@ -26,8 +26,6 @@ def make_psf_map(psf, pointing, geom, max_offset, exposure_map=None):
     geom : `~gammapy.maps.Geom`
         the map geom to be used. It provides the target geometry.
         rad and true energy axes should be given in this specific order.
-    max_offset : `~astropy.coordinates.Angle`
-        maximum offset w.r.t. fov center
     exposure_map : `~gammapy.maps.Map`, optional
         the associated exposure map.
         default is None
@@ -41,23 +39,25 @@ def make_psf_map(psf, pointing, geom, max_offset, exposure_map=None):
     energy = energy_axis.center
 
     rad_axis = geom.get_axis_by_name("theta")
-    rad = Angle(rad_axis.center, unit=rad_axis.unit)
+    rad = rad_axis.center
 
     # Compute separations with pointing position
-    separations = pointing.separation(geom.to_image().get_coord().skycoord)
-    valid = np.where(separations < max_offset)
+    offset = geom.separation(pointing)
 
     # Compute PSF values
-    psf_values = psf.evaluate(offset=separations[valid], energy=energy, rad=rad)
-
-    # Re-order axes to be consistent with expected geometry
-    psf_values = np.transpose(psf_values, axes=(2, 0, 1))
+    # TODO: allow broadcasting in PSF3D.evaluate()
+    psf_values = psf._interpolate(
+        (
+            rad[:, np.newaxis, np.newaxis],
+            offset,
+            energy[:, np.newaxis, np.newaxis, np.newaxis],
+        )
+    )
 
     # TODO: this probably does not ensure that probability is properly normalized in the PSFMap
     # Create Map and fill relevant entries
-    psfmap = Map.from_geom(geom, unit="sr-1")
-    psfmap.data[:, :, valid[0], valid[1]] += psf_values.to_value(psfmap.unit)
-
+    data = psf_values.to_value("sr-1")
+    psfmap = Map.from_geom(geom, data=data, unit="sr-1")
     return PSFMap(psfmap, exposure_map)
 
 
@@ -124,15 +124,6 @@ class PSFMap:
             raise ValueError("Incorrect theta axis position in input Map")
 
         self.psf_map = psf_map
-
-        if exposure_map is not None and exposure_map.geom.ndim == 3:
-            energy_axis = psf_map.geom.get_axis_by_name("energy")
-            rad_axis = psf_map.geom.get_axis_by_name("theta")
-            geom_image = exposure_map.geom.to_image()
-            geom = geom_image.to_cube([rad_axis.squash(), energy_axis])
-            data = exposure_map.data[:, np.newaxis, :, :]
-            exposure_map = Map.from_geom(geom=geom, data=data, unit=exposure_map.unit)
-
         self.exposure_map = exposure_map
 
     @classmethod
@@ -246,8 +237,18 @@ class PSFMap:
         energies = self.psf_map.geom.axes[1].center
         rad = self.psf_map.geom.axes[0].center
 
+        if self.exposure_map is not None:
+            exposure_3d = self.exposure_map.slice_by_idx({"theta": 0})
+            coords = {"skycoord": position, "energy": energies.reshape((-1, 1, 1))}
+            data = exposure_3d.interp_by_coord(coords).squeeze()
+            exposure = data * self.exposure_map.unit
+        else:
+            exposure = None
+
         # Beware. Need to revert rad and energies to follow the TablePSF scheme.
-        return EnergyDependentTablePSF(energy=energies, rad=rad, psf_value=psf_values.T)
+        return EnergyDependentTablePSF(
+            energy=energies, rad=rad, psf_value=psf_values.T, exposure=exposure
+        )
 
     def get_psf_kernel(self, position, geom, max_radius=None, factor=4):
         """Returns a PSF kernel at the given position.
@@ -271,6 +272,8 @@ class PSFMap:
             the resulting kernel
         """
         table_psf = self.get_energy_dependent_table_psf(position)
+        if max_radius is None:
+            max_radius = np.max(table_psf.rad)
         return PSFKernel.from_table_psf(table_psf, geom, max_radius, factor)
 
     def containment_radius_map(self, energy, fraction=0.68):
@@ -298,36 +301,157 @@ class PSFMap:
 
         return m
 
-    def stack(self, other):
-        """Stack PSFMap with another one.
+    def stack(self, other, weights=None):
+        """Stack PSFMap with another one in place.
 
         Parameters
         ----------
         other : `~gammapy.cube.PSFMap`
             the psfmap to be stacked with this one.
 
-        Returns
-        -------
-        new : `~gammapy.cube.PSFMap`
-            the stacked psfmap
         """
         if self.exposure_map is None or other.exposure_map is None:
             raise ValueError("Missing exposure map for PSFMap.stack")
 
-        geom_image = other.psf_map.geom.to_image()
-        coords = geom_image.get_coord()
+        cutout_info = other.psf_map.geom.cutout_info
 
-        # compute indices in the map to stack in
-        idx_x, idx_y = self.psf_map.geom.to_image().coord_to_idx(coords)
-        slice_ = (Ellipsis, idx_y, idx_x)
+        if cutout_info is not None:
+            slices = cutout_info["parent-slices"]
+            parent_slices = Ellipsis, slices[0], slices[1]
+        else:
+            parent_slices = None
+
+        self.psf_map.data[parent_slices] *= self.exposure_map.data[parent_slices]
+        self.psf_map.stack(other.psf_map * other.exposure_map.data, weights=weights)
+
+        # stack exposure map
+        self.exposure_map.stack(other.exposure_map, weights=weights)
 
-        self.psf_map.data[slice_] *= self.exposure_map.data[slice_]
-        self.psf_map.data[slice_] += other.psf_map.data * other.exposure_map.data
-        self.exposure_map.data[slice_] += other.exposure_map.data
         with np.errstate(invalid="ignore"):
-            self.psf_map.data[slice_] /= self.exposure_map.data[slice_]
+            self.psf_map.data[parent_slices] /= self.exposure_map.data[parent_slices]
             self.psf_map.data = np.nan_to_num(self.psf_map.data)
 
     def copy(self):
         """Copy PSFMap"""
         return deepcopy(self)
+
+    @classmethod
+    def from_geom(cls, geom):
+        """Create psf map from geom.
+
+        Parameters
+        ----------
+        geom : `Geom`
+            PSF map geometry.
+
+        Returns
+        -------
+        psf_map : `PSFMap`
+            Point spread function map.
+        """
+        geom_exposure_psf = geom.squash(axis="theta")
+        exposure_psf = Map.from_geom(geom_exposure_psf, unit="m2 s")
+        psf_map = Map.from_geom(geom, unit="sr-1")
+        return cls(psf_map, exposure_psf)
+
+    def sample_coord(self, map_coord, random_state=0):
+        """Apply PSF corrections on the coordinates of a set of simulated events.
+
+        Parameters
+        ----------
+        map_coord : `~gammapy.maps.MapCoord` object.
+            Sequence of coordinates and energies of sampled events.
+        random_state : {int, 'random-seed', 'global-rng', `~numpy.random.RandomState`}
+            Defines random number generator initialisation.
+            Passed to `~gammapy.utils.random.get_random_state`.
+
+        Returns
+        -------
+        corr_coord : `~gammapy.maps.MapCoord` object.
+            Sequence of PSF-corrected coordinates of the input map_coord map.
+        """
+
+        random_state = get_random_state(random_state)
+        rad_axis = self.psf_map.geom.get_axis_by_name("theta")
+
+        coord = {
+            "skycoord": map_coord.skycoord.reshape(-1, 1),
+            "energy": map_coord["energy"].reshape(-1, 1),
+            "theta": rad_axis.center,
+        }
+
+        pdf = self.psf_map.interp_by_coord(coord)
+
+        sample_pdf = InverseCDFSampler(pdf, axis=1, random_state=random_state)
+        pix_coord = sample_pdf.sample_axis()
+        separation = rad_axis.pix_to_coord(pix_coord)
+
+        position_angle = random_state.uniform(360, size=len(map_coord.lon)) * u.deg
+
+        event_positions = map_coord.skycoord.directional_offset_by(
+            position_angle=position_angle, separation=separation
+        )
+        return MapCoord.create(
+            {"skycoord": event_positions, "energy": map_coord["energy"]}
+        )
+
+    @classmethod
+    def from_energy_dependent_table_psf(cls, table_psf):
+        """Create PSF map from table PSF object.
+
+        Helper function to create an allsky PSF map from
+        table PSF, which does not depend on position.
+
+        Parameters
+        ----------
+        table_psf : `EnergyDependentTablePSF`
+            Table PSF
+
+        Returns
+        -------
+        psf_map : `PSFMap`
+            Point spread function map.
+        """
+        energy_axis = MapAxis.from_nodes(table_psf.energy, name="energy", interp="log")
+        rad_axis = MapAxis.from_nodes(table_psf.rad, name="theta")
+
+        geom = WcsGeom.create(
+            npix=(4, 2), proj="CAR", binsz=180, axes=[rad_axis, energy_axis]
+        )
+        coords = geom.get_coord()
+
+        # TODO: support broadcasting in .evaluate()
+        data = table_psf._interpolate((coords["energy"], coords["theta"])).to_value(
+            "sr-1"
+        )
+        psf_map = Map.from_geom(geom, data=data, unit="sr-1")
+
+        geom_exposure = geom.squash(axis="theta")
+
+        data = table_psf.exposure.reshape((-1, 1, 1, 1))
+
+        exposure_map = Map.from_geom(geom_exposure, unit="cm2 s")
+        exposure_map.quantity += data
+        return cls(psf_map=psf_map, exposure_map=exposure_map)
+
+    def cutout(self, position, width, mode="trim"):
+        """Cutout map psf map.
+
+        Parameters
+        ----------
+        position : `~astropy.coordinates.SkyCoord`
+            Center position of the cutout region.
+        width : tuple of `~astropy.coordinates.Angle`
+            Angular sizes of the region in (lon, lat) in that specific order.
+            If only one value is passed, a square region is extracted.
+        mode : {'trim', 'partial', 'strict'}
+            Mode option for Cutout2D, for details see `~astropy.nddata.utils.Cutout2D`.
+
+        Returns
+        -------
+        cutout : `PSFMap`
+            Cutout psf map.
+        """
+        psf_map = self.psf_map.cutout(position, width, mode)
+        exposure_map = self.exposure_map.cutout(position, width, mode)
+        return self.__class__(psf_map=psf_map, exposure_map=exposure_map)
